searchData={"items":[{"type":"module","title":"Drops.Casters","doc":"Drops.Casters is a module that provides functions for casting values\nfrom one type to another.\n\nThis module is the default caster module used by the Drops.Types.Map.DSL.cast function.","ref":"Drops.Casters.html"},{"type":"function","title":"Drops.Casters.cast/3","doc":"Casts a string into an integer","ref":"Drops.Casters.html#cast/3"},{"type":"function","title":"Casting a string into an integer - Drops.Casters.cast/3","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{required(:age) => cast(:string) |> type(:integer)}\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{age: \"20\"})\n    {:ok, %{age: 20}}","ref":"Drops.Casters.html#cast/3-casting-a-string-into-an-integer"},{"type":"function","title":"Casting a string into a float - Drops.Casters.cast/3","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{required(:num) => cast(:string) |> type(:float)}\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{num: \"20.5\"})\n    {:ok, %{num: 20.5}}","ref":"Drops.Casters.html#cast/3-casting-a-string-into-a-float"},{"type":"function","title":"Casting an integer into a string - Drops.Casters.cast/3","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{required(:id) => cast(:integer) |> type(:string)}\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{id: 312})\n    {:ok, %{id: \"312\"}}","ref":"Drops.Casters.html#cast/3-casting-an-integer-into-a-string"},{"type":"function","title":"Casting an integer into a date time - Drops.Casters.cast/3","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{required(:date) => cast(:integer) |> type(:date_time)}\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{date: 1614556800})\n    {:ok, %{date: ~U[2021-03-01 00:00:00Z]}}","ref":"Drops.Casters.html#cast/3-casting-an-integer-into-a-date-time"},{"type":"function","title":"Drops.Casters.cast/4","doc":"","ref":"Drops.Casters.html#cast/4"},{"type":"behaviour","title":"Drops.Contract","doc":"Drops.Contract can be used to extend your module with data validation capabilities.","ref":"Drops.Contract.html"},{"type":"callback","title":"Drops.Contract.conform/1","doc":"Validates the given `data` against the schema defined in the module.\n\nReturns `{:ok, validated_data}`.","ref":"Drops.Contract.html#c:conform/1"},{"type":"callback","title":"Examples - Drops.Contract.conform/1","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{\n    ...>       required(:name) => type(:string),\n    ...>       required(:age) => type(:integer)\n    ...>     }\n    ...>   end\n    ...> end\n    iex> {:error, errors} = UserContract.conform(\"oops\")\n    iex> Enum.map(errors, &to_string/1)\n    [\"must be a map\"]\n    iex> UserContract.conform(%{name: \"Jane\", age: 48})\n    {:ok, %{name: \"Jane\", age: 48}}\n    iex> {:error, errors} = UserContract.conform(%{name: \"Jane\", age: \"not an integer\"})\n    iex> Enum.map(errors, &to_string/1)\n    [\"age must be an integer\"]","ref":"Drops.Contract.html#c:conform/1-examples"},{"type":"callback","title":"Drops.Contract.conform/2","doc":"","ref":"Drops.Contract.html#c:conform/2"},{"type":"callback","title":"Drops.Contract.conform/3","doc":"","ref":"Drops.Contract.html#c:conform/3"},{"type":"macro","title":"Drops.Contract.rule/3","doc":"Define validation rules that are applied to the data validated by the schema.","ref":"Drops.Contract.html#rule/3"},{"type":"macro","title":"Examples - Drops.Contract.rule/3","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{\n    ...>       required(:email) => maybe(:string),\n    ...>       required(:login) => maybe(:string)\n    ...>     }\n    ...>   end\n    ...>\n    ...>   rule(:either_login_or_email, %{email: nil, login: nil}) do\n    ...>     {:error, \"email or login must be present\"}\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{email: \"jane@doe.org\", login: nil})\n    {:ok, %{email: \"jane@doe.org\", login: nil}}\n    iex> UserContract.conform(%{email: nil, login: \"jane\"})\n    {:ok, %{email: nil, login: \"jane\"}}\n    iex> {:error, errors} = UserContract.conform(%{email: nil, login: nil})\n    iex> Enum.map(errors, &to_string/1)\n    [\"email or login must be present\"]","ref":"Drops.Contract.html#rule/3-examples"},{"type":"macro","title":"Drops.Contract.schema/1","doc":"Define a default schema for the contract.","ref":"Drops.Contract.html#schema/1"},{"type":"macro","title":"Simple schema - Drops.Contract.schema/1","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema do\n    ...>     %{\n    ...>       required(:name) => type(:string),\n    ...>       required(:age) => type(:integer)\n    ...>     }\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{name: \"John\", age: 21})\n    {:ok, %{name: \"John\", age: 21}}","ref":"Drops.Contract.html#schema/1-simple-schema"},{"type":"macro","title":"Drops.Contract.schema/2","doc":"Define schemas for the contract.","ref":"Drops.Contract.html#schema/2"},{"type":"macro","title":"Nested atomized schema - Drops.Contract.schema/2","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema(atomize: true) do\n    ...>     %{\n    ...>       required(:user) => %{\n    ...>         required(:name) => type(:string, [:filled?]),\n    ...>         required(:age) => type(:integer),\n    ...>         required(:address) => %{\n    ...>           required(:city) => type(:string, [:filled?]),\n    ...>           required(:street) => type(:string, [:filled?]),\n    ...>           required(:zipcode) => type(:string, [:filled?])\n    ...>         }\n    ...>       }\n    ...>     }\n    ...>   end\n    ...> end\n    iex> {:error, errors} = UserContract.conform(%{\n    ...>  \"user\" => %{\n    ...>    \"name\" => \"John\",\n    ...>    \"age\" => 21,\n    ...>    \"address\" => %{\n    ...>      \"city\" => \"New York\",\n    ...>      \"street\" => \"\",\n    ...>      \"zipcode\" => \"10001\"\n    ...>    }\n    ...>  }\n    ...> })\n    iex> Enum.map(errors, &to_string/1)\n    [\"user.address.street must be filled\"]\n    iex> UserContract.conform(%{\n    ...>  \"user\" => %{\n    ...>    \"name\" => \"John\",\n    ...>    \"age\" => 21,\n    ...>    \"address\" => %{\n    ...>      \"city\" => \"New York\",\n    ...>      \"street\" => \"Central Park\",\n    ...>      \"zipcode\" => \"10001\"\n    ...>    }\n    ...>  }\n    ...> })\n    {:ok,\n     %{\n       user: %{\n         name: \"John\",\n         address: %{city: \"New York\", street: \"Central Park\", zipcode: \"10001\"},\n         age: 21\n       }\n     }}","ref":"Drops.Contract.html#schema/2-nested-atomized-schema"},{"type":"macro","title":"Reusing schemas - Drops.Contract.schema/2","doc":"iex> defmodule UserContract do\n    ...>   use Drops.Contract\n    ...>\n    ...>   schema(:address) do\n    ...>     %{\n    ...>       required(:street) => string(:filled?),\n    ...>       required(:city) => string(:filled?),\n    ...>       required(:zip) => string(:filled?),\n    ...>       required(:country) => string(:filled?)\n    ...>     }\n    ...>   end\n    ...>\n    ...>   schema do\n    ...>     %{\n    ...>       required(:name) => string(),\n    ...>       required(:age) => integer(),\n    ...>       required(:address) => @schemas.address\n    ...>     }\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{\n    ...>   name: \"John\",\n    ...>   age: 21,\n    ...>   address: %{\n    ...>     street: \"Main St.\",\n    ...>     city: \"New York\",\n    ...>     zip: \"10001\",\n    ...>     country: \"USA\"\n    ...>   }\n    ...> })\n    {:ok,\n     %{\n       name: \"John\",\n       address: %{\n         zip: \"10001\",\n         street: \"Main St.\",\n         city: \"New York\",\n         country: \"USA\"\n       },\n       age: 21\n     }}\n    iex> {:error, errors} = UserContract.conform(%{\n    ...>   name: \"John\",\n    ...>   age: \"21\",\n    ...>   address: %{\n    ...>     street: \"Main St.\",\n    ...>     city: \"\",\n    ...>     zip: \"10001\",\n    ...>     country: \"USA\"\n    ...>   }\n    ...> })\n    iex> Enum.map(errors, &to_string/1)\n    [\"address.city must be filled\", \"age must be an integer\"]","ref":"Drops.Contract.html#schema/2-reusing-schemas"},{"type":"macro","title":"Drops.Contract.schema/3","doc":"","ref":"Drops.Contract.html#schema/3"},{"type":"module","title":"Drops.Predicates","doc":"Drops.Predicates is a module that provides validation functions that can be used as the\ntype constraints.","ref":"Drops.Predicates.html"},{"type":"function","title":"Drops.Predicates.empty?/1","doc":"Checks if a given input is empty","ref":"Drops.Predicates.html#empty?/1"},{"type":"function","title":"Examples - Drops.Predicates.empty?/1","doc":"iex> Drops.Predicates.empty?(\"hello\")\n    false\n    iex> Drops.Predicates.empty?(\"\")\n    true\n    iex> Drops.Predicates.empty?([\"hello\", \"world\"])\n    false\n    iex> Drops.Predicates.empty?(%{hello: \"world\"})\n    false\n    iex> Drops.Predicates.empty?(%{})\n    true","ref":"Drops.Predicates.html#empty?/1-examples"},{"type":"function","title":"Drops.Predicates.eql?/2","doc":"Checks if a given value is equal to another value","ref":"Drops.Predicates.html#eql?/2"},{"type":"function","title":"Examples - Drops.Predicates.eql?/2","doc":"iex> Drops.Predicates.eql?(\"hello\", \"hello\")\n    true\n    iex> Drops.Predicates.eql?(\"hello\", \"world\")\n    false","ref":"Drops.Predicates.html#eql?/2-examples"},{"type":"function","title":"Drops.Predicates.even?/1","doc":"Checks if a given integer is even","ref":"Drops.Predicates.html#even?/1"},{"type":"function","title":"Examples - Drops.Predicates.even?/1","doc":"iex> Drops.Predicates.even?(4)\n    true\n    iex> Drops.Predicates.even?(7)\n    false","ref":"Drops.Predicates.html#even?/1-examples"},{"type":"function","title":"Drops.Predicates.excludes?/2","doc":"Checks if a given element is not included in a given list","ref":"Drops.Predicates.html#excludes?/2"},{"type":"function","title":"Examples - Drops.Predicates.excludes?/2","doc":"iex> Drops.Predicates.excludes?(1, [1, 2, 3])\n    false\n    iex> Drops.Predicates.excludes?(4, [1, 2, 3])\n    true","ref":"Drops.Predicates.html#excludes?/2-examples"},{"type":"function","title":"Drops.Predicates.filled?/1","doc":"Checks if a given input is not empty","ref":"Drops.Predicates.html#filled?/1"},{"type":"function","title":"Examples - Drops.Predicates.filled?/1","doc":"iex> Drops.Predicates.filled?(\"hello\")\n    true\n    iex> Drops.Predicates.filled?(\"\")\n    false\n    iex> Drops.Predicates.filled?([\"hello\", \"world\"])\n    true\n    iex> Drops.Predicates.filled?(%{hello: \"world\"})\n    true\n    iex> Drops.Predicates.filled?(%{})\n    false","ref":"Drops.Predicates.html#filled?/1-examples"},{"type":"function","title":"Drops.Predicates.gt?/2","doc":"Checks if a given input is greater than another value","ref":"Drops.Predicates.html#gt?/2"},{"type":"function","title":"Examples - Drops.Predicates.gt?/2","doc":"iex> Drops.Predicates.gt?(2, 1)\n    false\n    iex> Drops.Predicates.gt?(1, 2)\n    true","ref":"Drops.Predicates.html#gt?/2-examples"},{"type":"function","title":"Drops.Predicates.gteq?/2","doc":"Checks if a given value is greater than or equal to another value","ref":"Drops.Predicates.html#gteq?/2"},{"type":"function","title":"Examples - Drops.Predicates.gteq?/2","doc":"iex> Drops.Predicates.gteq?(2, 1)\n    false\n    iex> Drops.Predicates.gteq?(1, 1)\n    true\n    iex> Drops.Predicates.gteq?(1, 2)\n    true","ref":"Drops.Predicates.html#gteq?/2-examples"},{"type":"function","title":"Drops.Predicates.in?/2","doc":"Checks if a given element is included in a given list","ref":"Drops.Predicates.html#in?/2"},{"type":"function","title":"Examples - Drops.Predicates.in?/2","doc":"iex> Drops.Predicates.in?([1, 2, 3], 2)\n    true\n    iex> Drops.Predicates.in?([1, 2, 3], 4)\n    false","ref":"Drops.Predicates.html#in?/2-examples"},{"type":"function","title":"Drops.Predicates.includes?/2","doc":"Checks if a given element is included in a given list","ref":"Drops.Predicates.html#includes?/2"},{"type":"function","title":"Examples - Drops.Predicates.includes?/2","doc":"iex> Drops.Predicates.includes?(1, [1, 2, 3])\n    true\n    iex> Drops.Predicates.includes?(4, [1, 2, 3])\n    false","ref":"Drops.Predicates.html#includes?/2-examples"},{"type":"function","title":"Drops.Predicates.lt?/2","doc":"Checks if a given input is less than another value","ref":"Drops.Predicates.html#lt?/2"},{"type":"function","title":"Examples - Drops.Predicates.lt?/2","doc":"iex> Drops.Predicates.lt?(2, 1)\n    true\n    iex> Drops.Predicates.lt?(1, 2)\n    false","ref":"Drops.Predicates.html#lt?/2-examples"},{"type":"function","title":"Drops.Predicates.lteq?/2","doc":"Checks if a given input is less than or equal to another value","ref":"Drops.Predicates.html#lteq?/2"},{"type":"function","title":"Examples - Drops.Predicates.lteq?/2","doc":"iex> Drops.Predicates.lteq?(2, 1)\n    true\n    iex> Drops.Predicates.lteq?(1, 1)\n    true\n    iex> Drops.Predicates.lteq?(1, 2)\n    false","ref":"Drops.Predicates.html#lteq?/2-examples"},{"type":"function","title":"Drops.Predicates.match?/2","doc":"Checks if a given string matches a given regular expression","ref":"Drops.Predicates.html#match?/2"},{"type":"function","title":"Examples - Drops.Predicates.match?/2","doc":"iex> Drops.Predicates.match?(~r/hello/, \"hello world\")\n    true\n    iex> Drops.Predicates.match?(~r/hello/, \"world\")\n    false","ref":"Drops.Predicates.html#match?/2-examples"},{"type":"function","title":"Drops.Predicates.max_size?/2","doc":"Checks if a given map, list or string size is less than or equal to a given size","ref":"Drops.Predicates.html#max_size?/2"},{"type":"function","title":"Examples - Drops.Predicates.max_size?/2","doc":"iex> Drops.Predicates.max_size?(2, \"a\")\n    true\n    iex> Drops.Predicates.max_size?(2, \"ab\")\n    true\n    iex> Drops.Predicates.max_size?(2, \"abc\")\n    false\n    iex> Drops.Predicates.max_size?(2, [1, 2])\n    true\n    iex> Drops.Predicates.max_size?(2, [1, 2, 3])\n    false\n    iex> Drops.Predicates.max_size?(2, %{a: 1, b: 2})\n    true\n    iex> Drops.Predicates.max_size?(2, %{a: 1, b: 2, c: 3})\n    false","ref":"Drops.Predicates.html#max_size?/2-examples"},{"type":"function","title":"Drops.Predicates.min_size?/2","doc":"Checks if a given map, list or string size is greater than or equal to a given size","ref":"Drops.Predicates.html#min_size?/2"},{"type":"function","title":"Examples - Drops.Predicates.min_size?/2","doc":"iex> Drops.Predicates.min_size?(2, \"ab\")\n    true\n    iex> Drops.Predicates.min_size?(2, \"abc\")\n    true\n    iex> Drops.Predicates.min_size?(2, \"a\")\n    false\n    iex> Drops.Predicates.min_size?(2, [1, 2])\n    true\n    iex> Drops.Predicates.min_size?(2, [1])\n    false\n    iex> Drops.Predicates.min_size?(2, %{a: 1, b: 2})\n    true\n    iex> Drops.Predicates.min_size?(2, %{a: 1})\n    false","ref":"Drops.Predicates.html#min_size?/2-examples"},{"type":"function","title":"Drops.Predicates.not_eql?/2","doc":"Checks if a given value is not equal to another value","ref":"Drops.Predicates.html#not_eql?/2"},{"type":"function","title":"Examples - Drops.Predicates.not_eql?/2","doc":"iex> Drops.Predicates.not_eql?(\"hello\", \"hello\")\n    false\n    iex> Drops.Predicates.not_eql?(\"hello\", \"world\")\n    true","ref":"Drops.Predicates.html#not_eql?/2-examples"},{"type":"function","title":"Drops.Predicates.not_in?/2","doc":"Checks if a given element is not included in a given list","ref":"Drops.Predicates.html#not_in?/2"},{"type":"function","title":"Examples - Drops.Predicates.not_in?/2","doc":"iex> Drops.Predicates.not_in?([1, 2, 3], 2)\n    false\n    iex> Drops.Predicates.not_in?([1, 2, 3], 4)\n    true","ref":"Drops.Predicates.html#not_in?/2-examples"},{"type":"function","title":"Drops.Predicates.odd?/1","doc":"Checks if a given integer is odd","ref":"Drops.Predicates.html#odd?/1"},{"type":"function","title":"Examples - Drops.Predicates.odd?/1","doc":"iex> Drops.Predicates.odd?(4)\n    false\n    iex> Drops.Predicates.odd?(7)\n    true","ref":"Drops.Predicates.html#odd?/1-examples"},{"type":"function","title":"Drops.Predicates.size?/2","doc":"Checks if a given list, map or string size is equal to a given size","ref":"Drops.Predicates.html#size?/2"},{"type":"function","title":"Examples - Drops.Predicates.size?/2","doc":"iex> Drops.Predicates.size?(2, \"ab\")\n    true\n    iex> Drops.Predicates.size?(2, \"abc\")\n    false\n    iex> Drops.Predicates.size?(2, [1, 2])\n    true\n    iex> Drops.Predicates.size?(2, [1, 2, 3])\n    false\n    iex> Drops.Predicates.size?(2, %{a: 1, b: 2})\n    true\n    iex> Drops.Predicates.size?(2, %{a: 1, b: 2, c: 3})\n    false","ref":"Drops.Predicates.html#size?/2-examples"},{"type":"function","title":"Drops.Predicates.type?/2","doc":"Checks if a given value matches a given type identifier","ref":"Drops.Predicates.html#type?/2"},{"type":"function","title":"Examples - Drops.Predicates.type?/2","doc":"iex> Drops.Predicates.type?(:nil, nil)\n    true\n    iex> Drops.Predicates.type?(:any, nil)\n    true\n    iex> Drops.Predicates.type?(:any, \"foo\")\n    true\n    iex> Drops.Predicates.type?(:atom, :hello)\n    true\n    iex> Drops.Predicates.type?(:boolean, true)\n    true\n    iex> Drops.Predicates.type?(:boolean, false)\n    true\n    iex> Drops.Predicates.type?(:string, \"hello\")\n    true\n    iex> Drops.Predicates.type?(:integer, 1)\n    true\n    iex> Drops.Predicates.type?(:float, 1.2)\n    true\n    iex> Drops.Predicates.type?(:map, %{})\n    true\n    iex> Drops.Predicates.type?(:date_time, DateTime.utc_now())\n    true","ref":"Drops.Predicates.html#type?/2-examples"},{"type":"behaviour","title":"Drops.Validator.Messages.Backend","doc":"Messages Backends are used to generate error messages from error results.","ref":"Drops.Validator.Messages.Backend.html"},{"type":"behaviour","title":"Examples - Drops.Validator.Messages.Backend","doc":"iex> defmodule MyBackend do\n    ...>   use Drops.Validator.Messages.Backend\n    ...>\n    ...>   def text(:type?, type, input) do\n    ...>     \"#{inspect(input)} received but it must be a #{type}\"\n    ...>   end\n    ...>\n    ...>   def text(:filled?, _input) do\n    ...>     \"cannot be empty\"\n    ...>   end\n    ...> end\n    iex> defmodule UserContract do\n    ...>   use Drops.Contract, message_backend: MyBackend\n    ...>\n    ...>   schema do\n    ...>     %{\n    ...>       required(:name) => string(:filled?),\n    ...>       required(:email) => string(:filled?)\n    ...>     }\n    ...>   end\n    ...> end\n    iex> UserContract.conform(%{name: \"\", email: 312})\n    {:error,\n      [\n        %Drops.Validator.Messages.Error.Type{\n          path: [:email],\n          text: \"312 received but it must be a string\",\n          meta: %{args: [:string, 312], predicate: :type?}\n        },\n        %Drops.Validator.Messages.Error.Type{\n          path: [:name],\n          text: \"cannot be empty\",\n          meta: %{args: [\"\"], predicate: :filled?}\n        }\n      ]\n    }","ref":"Drops.Validator.Messages.Backend.html#module-examples"},{"type":"callback","title":"Drops.Validator.Messages.Backend.text/2","doc":"","ref":"Drops.Validator.Messages.Backend.html#c:text/2"},{"type":"callback","title":"Drops.Validator.Messages.Backend.text/3","doc":"","ref":"Drops.Validator.Messages.Backend.html#c:text/3"},{"type":"module","title":"Drops.Types","doc":"Drops.Types is a module that provides functions for creating type structs\nfrom DSL's type specs represented by plain tuples.","ref":"Drops.Types.html"},{"type":"function","title":"Drops.Types.from_spec/2","doc":"","ref":"Drops.Types.html#from_spec/2"},{"type":"function","title":"Drops.Types.infer_constraints/2","doc":"","ref":"Drops.Types.html#infer_constraints/2"},{"type":"function","title":"Drops.Types.infer_primitive/2","doc":"","ref":"Drops.Types.html#infer_primitive/2"},{"type":"function","title":"Drops.Types.predicate/1","doc":"","ref":"Drops.Types.html#predicate/1"},{"type":"function","title":"Drops.Types.predicate/2","doc":"","ref":"Drops.Types.html#predicate/2"},{"type":"module","title":"Drops.Types.Cast","doc":"Drops.Types.Cast is a struct that represents a cast type with input and output types.","ref":"Drops.Types.Cast.html"},{"type":"module","title":"Examples - Drops.Types.Cast","doc":"iex> Drops.Types.from_spec(\n    ...>   {:cast, {{:type, {:integer, []}}, {:type, {:date_time, []}}, [:miliseconds]}},\n    ...>   []\n    ...> )\n    %Drops.Types.Cast{\n      input_type: %Drops.Types.Type{\n        primitive: :integer,\n        constraints: [predicate: {:type?, :integer}]\n      },\n      output_type: %Drops.Types.Type{\n        primitive: :date_time,\n        constraints: [predicate: {:type?, :date_time}]\n      },\n      opts: [:miliseconds]\n    }","ref":"Drops.Types.Cast.html#module-examples"},{"type":"module","title":"Drops.Types.List","doc":"Drops.Types.List is a struct that represents a list type with a member type and optional\nconstraints.","ref":"Drops.Types.List.html"},{"type":"module","title":"Examples - Drops.Types.List","doc":"iex> Drops.Types.from_spec({:type, {:list, []}}, [])\n    %Drops.Types.Type{primitive: :list, constraints: [predicate: {:type?, :list}]}\n\n    iex> Drops.Types.from_spec({:type, {:list, {:type, {:integer, []}}}}, [])\n    %Drops.Types.List{\n      primitive: :list,\n      constraints: [predicate: {:type?, :list}],\n      member_type: %Drops.Types.Type{\n        primitive: :integer,\n        constraints: [predicate: {:type?, :integer}]\n      }\n    }","ref":"Drops.Types.List.html#module-examples"},{"type":"module","title":"Drops.Types.Map","doc":"Drops.Types.Map is a struct that represents a map type with optional constraints.","ref":"Drops.Types.Map.html"},{"type":"module","title":"Examples - Drops.Types.Map","doc":"iex> Drops.Types.from_spec({:type, {:map, []}}, [])\n    %Drops.Types.Type{primitive: :map, constraints: [predicate: {:type?, :map}]}\n\n    iex> Drops.Types.from_spec(%{\n    ...>   {:required, :name} => {:type, {:string, []}},\n    ...>   {:optional, :age} => {:type, {:integer, []}}\n    ...> }, [])\n    %Drops.Types.Map{\n      primitive: :map,\n      constraints: [predicate: {:type?, :map}],\n      keys: [\n        %Drops.Types.Map.Key{\n          path: [:age],\n          presence: :optional,\n          type: %Drops.Types.Type{\n            primitive: :integer,\n            constraints: [predicate: {:type?, :integer}]\n          }\n        },\n        %Drops.Types.Map.Key{\n          path: [:name],\n          presence: :required,\n          type: %Drops.Types.Type{\n            primitive: :string,\n            constraints: [predicate: {:type?, :string}]\n          }\n        }\n      ],\n      atomize: false\n    }","ref":"Drops.Types.Map.html#module-examples"},{"type":"function","title":"Drops.Types.Map.atomize/3","doc":"","ref":"Drops.Types.Map.html#atomize/3"},{"type":"type","title":"Drops.Types.Map.t/0","doc":"","ref":"Drops.Types.Map.html#t:t/0"},{"type":"module","title":"Drops.Types.Map.DSL","doc":"DSL functions for defining map key and value type specifications.\n\nFunctions from this module are typically used via Drops.Contract.schema/1","ref":"Drops.Types.Map.DSL.html"},{"type":"function","title":"Drops.Types.Map.DSL.any/0","doc":"Returns :any type specification.","ref":"Drops.Types.Map.DSL.html#any/0"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.any/0","doc":"any()","ref":"Drops.Types.Map.DSL.html#any/0-examples"},{"type":"function","title":"Drops.Types.Map.DSL.boolean/0","doc":"Returns a boolean type specification.","ref":"Drops.Types.Map.DSL.html#boolean/0"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.boolean/0","doc":"# a boolean with no constraints\n    boolean()","ref":"Drops.Types.Map.DSL.html#boolean/0-examples"},{"type":"function","title":"Drops.Types.Map.DSL.cast/2","doc":"Returns a type cast specification.","ref":"Drops.Types.Map.DSL.html#cast/2"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.cast/2","doc":"# cast a string to an integer\n    cast(:string) |> integer()\n\n    # cast a string to an integer with additional constraints\n    cast(string(match?: ~r/\\d+/])) |> integer()","ref":"Drops.Types.Map.DSL.html#cast/2-examples"},{"type":"function","title":"Drops.Types.Map.DSL.float/0","doc":"Returns a float type specification.","ref":"Drops.Types.Map.DSL.html#float/0"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.float/0","doc":"# a float with no constraints\n    float()","ref":"Drops.Types.Map.DSL.html#float/0-examples"},{"type":"function","title":"Drops.Types.Map.DSL.float/1","doc":"Returns a float type specification with additional constraints.","ref":"Drops.Types.Map.DSL.html#float/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.float/1","doc":"# a float with constraints\n    float(gt?: 1.0)","ref":"Drops.Types.Map.DSL.html#float/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.float/2","doc":"","ref":"Drops.Types.Map.DSL.html#float/2"},{"type":"function","title":"Drops.Types.Map.DSL.integer/0","doc":"Returns an integer type specification.","ref":"Drops.Types.Map.DSL.html#integer/0"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.integer/0","doc":"# an integer with no constraints\n    integer()","ref":"Drops.Types.Map.DSL.html#integer/0-examples"},{"type":"function","title":"Drops.Types.Map.DSL.integer/1","doc":"Returns an integer type specification with additional constraints.","ref":"Drops.Types.Map.DSL.html#integer/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.integer/1","doc":"# an integer with constraints\n    integer(:even?)\n\n    # an integer with multiple constraints\n    integer([:even?, gt?: 100])","ref":"Drops.Types.Map.DSL.html#integer/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.integer/2","doc":"","ref":"Drops.Types.Map.DSL.html#integer/2"},{"type":"function","title":"Drops.Types.Map.DSL.list/1","doc":"Returns a list type specification.","ref":"Drops.Types.Map.DSL.html#list/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.list/1","doc":"# a list with a specified member type\n    list(:string)\n\n    # a list with a specified sum member type\n    list([:string, :integer])","ref":"Drops.Types.Map.DSL.html#list/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.list/2","doc":"Returns a list type specification with a constrained member type","ref":"Drops.Types.Map.DSL.html#list/2"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.list/2","doc":"# a list with a specified member type\n    list(:string, [:filled?])\n\n    list(:integer, [gt?: 18])","ref":"Drops.Types.Map.DSL.html#list/2-examples"},{"type":"function","title":"Drops.Types.Map.DSL.map/0","doc":"Returns a map type specification.","ref":"Drops.Types.Map.DSL.html#map/0"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.map/0","doc":"# a map with no constraints\n    map()","ref":"Drops.Types.Map.DSL.html#map/0-examples"},{"type":"function","title":"Drops.Types.Map.DSL.map/1","doc":"Returns a map type specification with additional constraints.","ref":"Drops.Types.Map.DSL.html#map/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.map/1","doc":"# a map with constraints\n    map(min_size?: 2)","ref":"Drops.Types.Map.DSL.html#map/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.maybe/1","doc":"Returns a maybe type specification.","ref":"Drops.Types.Map.DSL.html#maybe/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.maybe/1","doc":"# either a nil or a string\n    maybe(:string)","ref":"Drops.Types.Map.DSL.html#maybe/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.maybe/2","doc":"Returns a maybe type specification with additional constraints.","ref":"Drops.Types.Map.DSL.html#maybe/2"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.maybe/2","doc":"# either a nil or a non-empty string\n    maybe(:string, [:filled?])","ref":"Drops.Types.Map.DSL.html#maybe/2-examples"},{"type":"function","title":"Drops.Types.Map.DSL.optional/1","doc":"Returns an optional key specification.","ref":"Drops.Types.Map.DSL.html#optional/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.optional/1","doc":"%{\n      optional(:age) => type(:integer)\n    }","ref":"Drops.Types.Map.DSL.html#optional/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.required/1","doc":"Returns a required key specification.","ref":"Drops.Types.Map.DSL.html#required/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.required/1","doc":"%{\n      required(:email) => type(:string)\n    }","ref":"Drops.Types.Map.DSL.html#required/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.string/0","doc":"Returns a string type specification.","ref":"Drops.Types.Map.DSL.html#string/0"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.string/0","doc":"# a string with no constraints\n    string()","ref":"Drops.Types.Map.DSL.html#string/0-examples"},{"type":"function","title":"Drops.Types.Map.DSL.string/1","doc":"Returns a string type specification with additional constraints.","ref":"Drops.Types.Map.DSL.html#string/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.string/1","doc":"# a string with constraints\n    string(:filled?)\n\n    # a string with multiple constraints\n    string([:filled?, max_length?: 255])","ref":"Drops.Types.Map.DSL.html#string/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.string/2","doc":"","ref":"Drops.Types.Map.DSL.html#string/2"},{"type":"function","title":"Drops.Types.Map.DSL.type/1","doc":"Returns a type specification.","ref":"Drops.Types.Map.DSL.html#type/1"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.type/1","doc":"# string\n    type(:string)\n\n    # either a nil or a string\n    type([:nil, :string])","ref":"Drops.Types.Map.DSL.html#type/1-examples"},{"type":"function","title":"Drops.Types.Map.DSL.type/2","doc":"Returns a type specification with additional constraints.","ref":"Drops.Types.Map.DSL.html#type/2"},{"type":"function","title":"Examples - Drops.Types.Map.DSL.type/2","doc":"# string with that must be filled\n    type(:string, [:filled?]),\n\n    # an integer that must be greater than 18\n    type(:integer, [gt?: 18])","ref":"Drops.Types.Map.DSL.html#type/2-examples"},{"type":"type","title":"Drops.Types.Map.DSL.type/0","doc":"","ref":"Drops.Types.Map.DSL.html#t:type/0"},{"type":"module","title":"Drops.Types.Map.Key","doc":"","ref":"Drops.Types.Map.Key.html"},{"type":"function","title":"Drops.Types.Map.Key.present?/2","doc":"","ref":"Drops.Types.Map.Key.html#present?/2"},{"type":"function","title":"Drops.Types.Map.Key.stringify/1","doc":"","ref":"Drops.Types.Map.Key.html#stringify/1"},{"type":"module","title":"Drops.Types.Sum","doc":"Drops.Types.Sum is a struct that represents a sum type with left and right types.","ref":"Drops.Types.Sum.html"},{"type":"module","title":"Examples - Drops.Types.Sum","doc":"iex> Drops.Types.from_spec([{:type, {:string, []}}, {:type, {:integer, []}}], [])\n    %Drops.Types.Sum{\n      left: %Drops.Types.Type{\n        primitive: :string,\n        constraints: [predicate: {:type?, :string}]\n      },\n      right: %Drops.Types.Type{\n        primitive: :integer,\n        constraints: [predicate: {:type?, :integer}]\n      },\n      opts: []\n    }","ref":"Drops.Types.Sum.html#module-examples"},{"type":"module","title":"Drops.Types.Type","doc":"Drops.Types.Type is a struct that represents a primitive type with optional constraints.","ref":"Drops.Types.Type.html"},{"type":"module","title":"Examples - Drops.Types.Type","doc":"iex> Drops.Types.from_spec({:type, {:string, []}}, [])\n    %Drops.Types.Type{\n      primitive: :string,\n      constraints: [predicate: {:type?, :string}]\n    }\n\n    iex> Drops.Types.from_spec({:type, {:string, [:filled?]}}, [])\n    %Drops.Types.Type{\n      primitive: :string,\n      constraints: {:and, [predicate: {:type?, :string}, predicate: {:filled?, []}]}\n    }","ref":"Drops.Types.Type.html#module-examples"},{"type":"extras","title":"Elixir Drops 💦","doc":"# Elixir Drops 💦\n[![CI](https://github.com/solnic/drops/actions/workflows/ci.yml/badge.svg)](https://github.com/solnic/drops/actions/workflows/ci.yml) [![Hex pm](https://img.shields.io/hexpm/v/drops.svg?style=flat)](https://hex.pm/packages/drops) [![hex.pm downloads](https://img.shields.io/hexpm/dt/drops.svg?style=flat)](https://hex.pm/packages/drops)\n\nElixir `Drops` is a collection of small modules that provide useful extensions and functions that can be used to work with data effectively.","ref":"readme.html"},{"type":"extras","title":"Installation - Elixir Drops 💦","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed by adding `drops` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:drops, \"~> 0.1.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc) and published on [HexDocs](https://hexdocs.pm). Once published, the docs can be found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Contracts - Elixir Drops 💦","doc":"You can use `Drops.Contract` to define data coercion and validation schemas with arbitrary validation rules.\n\nHere's an example of a simple `UserContract` which defines two required keys and expected types:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:name) => string(),\n      required(:email) => string()\n    }\n  end\nend\n\nUserContract.conform(%{name: \"Jane\", email: \"jane@doe.org\"})\n# {:ok, %{name: \"Jane\", email: \"jane@doe.org\"}}\n\n{:error, errors} = UserContract.conform(%{email: 312})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:email],\n#      text: \"must be a string\",\n#      meta: %{args: [:string, 312], predicate: :type?}\n#    },\n#    %Drops.Validator.Messages.Error.Key{\n#      path: [:name],\n#      text: \"key must be present\",\n#      meta: %{args: [:name], predicate: :has_key?}\n#    }\n#  ]}\n\nEnum.map(errors, &to_string/1)\n# [\"email must be a string\", \"name key must be present\"]\n\n{:error, errors} = UserContract.conform(%{name: \"Jane\", email: 312})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:email],\n#      text: \"must be a string\",\n#      meta: %{args: [:string, 312], predicate: :type?}\n#    }\n#  ]}\n\nEnum.map(errors, &to_string/1)\n# [\"email must be a string\"]\n```","ref":"readme.html#contracts"},{"type":"extras","title":"Schemas - Elixir Drops 💦","doc":"Contract's schemas are a powerful way of defining the exact shape of the data you expect to work with. They are used to validate **the structure** and **the values** of the input data. Using schemas, you can define which keys are required and whic are optional, the exact types of the values and any additional checks that have to be applied to the values.\n\n#","ref":"readme.html#schemas"},{"type":"extras","title":"Required and optional keys - Elixir Drops 💦","doc":"A schema must explicitly define which keys are required and which are optional. This is done by using `required` and `optional` functions. Here's an example:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      optional(:name) => string(),\n      required(:email) => string()\n    }\n  end\nend\n\nUserContract.conform(%{email: \"janedoe.org\"})\n# {:ok, %{email: \"janedoe.org\"}}\n\nUserContract.conform(%{name: \"Jane\", email: \"janedoe.org\"})\n# {:ok, %{name: \"Jane\", email: \"janedoe.org\"}}\n```\n\n#","ref":"readme.html#required-and-optional-keys"},{"type":"extras","title":"Types - Elixir Drops 💦","doc":"You can define the expected types of the values using `string`, `integer`, `float`, `boolean`, `atom`, `map`, `list`, `any` and `maybe` functions. Here's an example:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:name) => string(),\n      required(:age) => integer(),\n      required(:active) => boolean(),\n      required(:tags) => list(:string),\n      required(:settings) => map(:string),\n      required(:address) => maybe(:string)\n    }\n  end\nend\n```\n\n#","ref":"readme.html#types"},{"type":"extras","title":"Predicate checks - Elixir Drops 💦","doc":"You can define types that must meet additional requirements by using built-in predicates. Here's an example:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:name) => string(:filled?),\n      required(:age) => integer(gt?: 18)\n    }\n  end\nend\n\nUserContract.conform(%{name: \"Jane\", age: 21})\n# {:ok, %{name: \"Jane\", age: 21}}\n\nUserContract.conform(%{name: \"\", age: 21})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:name],\n#      text: \"must be filled\",\n#      meta: %{args: [\"\"], predicate: :filled?}\n#    }\n#  ]}\n\nUserContract.conform(%{name: \"Jane\", age: 12})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:age],\n#      text: \"must be greater than 18\",\n#      meta: %{args: [18, 12], predicate: :gt?}\n#    }\n#  ]}\n\n```\n\n#","ref":"readme.html#predicate-checks"},{"type":"extras","title":"Nested schemas - Elixir Drops 💦","doc":"Schemas can be nested, including complex cases like nested lists and maps. Here's an example:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:user) => %{\n        required(:name) => string(:filled?),\n        required(:age) => integer(),\n        required(:address) => %{\n          required(:city) => string(:filled?),\n          required(:street) => string(:filled?),\n          required(:zipcode) => string(:filled?)\n        },\n        required(:tags) =>\n          list(%{\n            required(:name) => string(:filled?),\n            required(:created_at) => integer()\n          })\n      }\n    }\n  end\nend\n\nUserContract.conform(%{\n  user: %{\n    name: \"Jane\",\n    age: 21,\n    address: %{\n      city: \"New York\",\n      street: \"Broadway\",\n      zipcode: \"10001\"\n    },\n    tags: [\n      %{name: \"foo\", created_at: 1_234_567_890},\n      %{name: \"bar\", created_at: 1_234_567_890}\n    ]\n  }\n})\n# {:ok,\n#   %{\n#     user: %{\n#       name: \"Jane\",\n#       address: %{city: \"New York\", street: \"Broadway\", zipcode: \"10001\"},\n#       age: 21,\n#       tags: [\n#         %{name: \"foo\", created_at: 1234567890},\n#         %{name: \"bar\", created_at: 1234567890}\n#       ]\n#     }\n#   }}\n\nUserContract.conform(%{\n  user: %{\n    name: \"Jane\",\n    age: 21,\n    address: %{\n      city: \"New York\",\n      street: \"Broadway\",\n      zipcode: \"\"\n    },\n    tags: [\n      %{name: \"foo\", created_at: 1_234_567_890},\n      %{name: \"bar\", created_at: nil}\n    ]\n  }\n})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:user, :address, :zipcode],\n#      text: \"must be filled\",\n#      meta: %{args: [\"\"], predicate: :filled?}\n#    },\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:user, :tags, 1, :created_at],\n#      text: \"must be an integer\",\n#      meta: %{args: [:integer, nil], predicate: :type?}\n#    }\n#  ]}\n```\n\n#","ref":"readme.html#nested-schemas"},{"type":"extras","title":"Type-safe casting - Elixir Drops 💦","doc":"You can define custom type casting functions that will be applied to the input data before it's validated. This is useful when you want to convert the input data to a different format, for example, when you want to convert a string to an integer. Here's an example:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:count) => cast(:string) |> integer(gt?: 0)\n    }\n  end\nend\n\nUserContract.conform(%{count: \"1\"})\n# {:ok, %{count: 1}}\n\nUserContract.conform(%{count: nil})\n#  [\n#    %Drops.Validator.Messages.Error.Caster{\n#      error: %Drops.Validator.Messages.Error.Type{\n#        path: [:count],\n#        text: \"must be a string\",\n#        meta: %{args: [:string, nil], predicate: :type?}\n#      }\n#    }\n#  ]}\n\nUserContract.conform(%{count: \"-1\"})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Type{\n#      path: [:count],\n#      text: \"must be greater than 0\",\n#      meta: %{args: [0, -1], predicate: :gt?}\n#    }\n#  ]}\n\n```\n\nIt's also possible to define a custom casting module and use it via `caster` option:\n\n```elixir\ndefmodule CustomCaster do\n  @spec cast(input_type :: atom(), output_type :: atom(), any, Keyword.t()) :: any()\n  def cast(:string, :string, value, _opts) do\n    String.downcase(value)\n  end\nend\n\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:text) => cast(:string, caster: CustomCaster) |> string()\n    }\n  end\nend\n\nUserContract.conform(%{text: \"HELLO\"})\n# {:ok, %{text: \"hello\"}}\n```\n\n#","ref":"readme.html#type-safe-casting"},{"type":"extras","title":"Atomized maps - Elixir Drops 💦","doc":"You can define a schema that will atomize the input map using `atomize: true` option. Only keys that you specified will be atomized, any unexpected key will be ignored. Here's an example:\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema(atomize: true) do\n    %{\n      required(:name) => string(),\n      required(:age) => integer(),\n      required(:tags) =>\n        list(%{\n          required(:name) => string()\n        })\n    }\n  end\nend\n\nUserContract.conform(%{\n  \"unexpected\" => \"value\",\n  \"this\" => \"should not be here\",\n  \"name\" => \"Jane\",\n  \"age\" => 21,\n  \"tags\" => [\n    %{\"name\" => \"red\"},\n    %{\"name\" => \"green\"},\n    %{\"name\" => \"blue\"}\n  ]\n})\n# {:ok,\n#  %{\n#    name: \"Jane\",\n#    age: 21,\n#    tags: [%{name: \"red\"}, %{name: \"green\"}, %{name: \"blue\"}]\n#  }}\n```","ref":"readme.html#atomized-maps"},{"type":"extras","title":"Rules - Elixir Drops 💦","doc":"You can define arbitrary rule functions using `rule` macro. These rules will be applied to the input data only if it passed schema validation. This way you can be sure that rules operate on data that's safe to work with.\n\n\nHere's an example how you could define a rule that checks if either email or login is provided:\n\n\n```elixir\ndefmodule UserContract do\n  use Drops.Contract\n\n  schema do\n    %{\n      required(:email) => maybe(:string),\n      required(:login) => maybe(:string)\n    }\n  end\n\n  rule(:either_login_or_email, %{email: nil, login: nil}) do\n    {:error, \"email or login must be provided\"}\n  end\nend\n\nUserContract.conform(%{email: \"jane@doe.org\", login: nil})\n# {:ok, %{email: \"jane@doe.org\", login: nil}}\n\nUserContract.conform(%{email: nil, login: \"jane\"})\n# {:ok, %{email: nil, login: \"jane\"}}\n\nUserContract.conform(%{email: nil, login: nil})\n# {:error,\n#  [\n#    %Drops.Validator.Messages.Error.Rule{\n#      path: [],\n#      text: \"email or login must be present\",\n#      meta: %{}\n#    }\n#  ]}\n```","ref":"readme.html#rules"},{"type":"extras","title":"CHANGELOG","doc":"","ref":"changelog.html"},{"type":"extras","title":"v0.1.1 - 2023-10-27 - CHANGELOG","doc":"#","ref":"changelog.html#v0-1-1-2023-10-27"},{"type":"extras","title":"Bug fixes - CHANGELOG","doc":"- [`Drops.Contract`] Rules are now correctly applied only to the root map (via #32)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.1.0 - 2023-10-25 - CHANGELOG","doc":"#","ref":"changelog.html#v0-1-0-2023-10-25"},{"type":"extras","title":"Features - CHANGELOG","doc":"- Added `Drops.Contract` module for defining validation schemas with additional rules\n- Added `Drops.Validator` module for running validation functions against input\n- Added `Drops.Validator.Messages.DefaultBackend` that's configured by default in contracts\n- Added `Drops.Types` module with the following built-in types:\n  - `Drops.Types.Type` - basic type\n  - `Drops.Types.List` - a list if member type\n  - `Drops.Types.Map` - a map with typed keys\n  - `Drops.Types.Sum` - a composition of two types\n  - `Drops.Types.Cast` - a type that defines from-to casting types and caster options\n- Added `Drops.Predicates` module which provides many common predicate functions like `filled?`, `gt?`, `size?` etc.\n- Added `Drops.Casters` module which provides common type casting functions that can be used with the built-in types","ref":"changelog.html#features"},{"type":"extras","title":"v0.0.0 - 2023-09-04 - CHANGELOG","doc":"Reserving the package name","ref":"changelog.html#v0-0-0-2023-09-04"}],"content_type":"text/markdown"}